package common

import (
	"fmt"
	"log/slog"
	"sync"
	"time"
)

var (
	recentScrobbles    = make(map[string]int64)
	mu                 sync.Mutex
	dedupWindow        = 300 // seconds (5 minutes)
	sessionlessSources = map[string]bool{
		"mpris": true,
		// add others here
	}
)

func ForceScrobbleTimestamp(user, track, album, source string, t time.Time) {
	key := fmt.Sprintf("%s:%s:%s:%s", user, track, album, source)
	mu.Lock()
	defer mu.Unlock()
	recentScrobbles[key] = t.Unix()
}

func IsDuplicateScrobble(user, trackID, sessionID, source string) bool {

	if sessionID == "" && !sessionlessSources[source] {
		slog.Warn("IsDuplicateScrobble called without sessionID — this should not happen",
			"user", user, "trackID", trackID, "source", source)
		return false
	}

	now := time.Now().Unix()
	key := fmt.Sprintf("%s:%s:%s:%s", user, trackID, sessionID, source)

	mu.Lock()
	defer mu.Unlock()

	ts, ok := recentScrobbles[key]
	withinWindow := ok && now-ts < int64(dedupWindow)

	slog.Debug("Deduplication check",
		"user", user,
		"trackID", trackID,
		"sessionID", sessionID,
		"source", source,
		"key", key,
		"now", now,
		"last_seen", ts,
		"within_window", withinWindow,
	)

	if withinWindow {
		return true
	}

	recentScrobbles[key] = now
	return false
}

func IsRecentScrobble(user, track, album, source string) bool {
	if track == "" || album == "" {
		slog.Debug("Time-based deduplication skipped: missing track or album",
			"user", user, "track", track, "album", album, "source", source)
		return false
	}

	now := time.Now().Unix()
	key := fmt.Sprintf("%s:%s:%s:%s", user, track, album, source)

	mu.Lock()
	defer mu.Unlock()

	ts, ok := recentScrobbles[key]
	withinWindow := ok && now-ts < int64(dedupWindow)

	slog.Debug("Time-based deduplication check",
		"user", user,
		"track", track,
		"album", album,
		"source", source,
		"key", key,
		"now", now,
		"last_seen", ts,
		"within_window", withinWindow,
	)

	if withinWindow {
		return true
	}

	recentScrobbles[key] = now
	return false
}
